@@ -56,11 +56,18 @@ void bd_close_all(void)
 
 void setLevelInfoToDefaults_BD_Ext(int width, int height)
 {
+  static GdCavesetData *caveset = NULL;		// used if no native cave loaded
   GdCave *cave = native_bd_level.cave;
 
+  if (caveset != NULL)
+    gd_caveset_data_free(caveset);
+
   if (cave != NULL)
     gd_cave_free(cave);
 
+  // get empty caveset, using default values
+  caveset = gd_caveset_data_new();
+
   // get empty cave, using default values
   cave = gd_cave_new();
 
@@ -81,7 +88,7 @@ void setLevelInfoToDefaults_BD_Ext(int width, int height)
   cave->selectable = TRUE;
   cave->intermission = FALSE;
 
-  native_bd_level.caveset = NULL;
+  native_bd_level.caveset = caveset;
   native_bd_level.cave = cave;
   native_bd_level.replay = NULL;
 
@@ -157,6 +164,7 @@ boolean LoadNativeLevel_BD(char *filename, int level_pos, boolean level_info_onl
   // set better initial cave speed (to set better native replay tape length)
   set_initial_cave_speed(native_bd_level.cave);
 
+  // caveset data for native levels managed separately -- do not free it!
   native_bd_level.caveset = gd_caveset_data;
   native_bd_level.loaded_from_caveset = TRUE;
 
@@ -286,7 +294,7 @@ static void UpdateGameDoorValues_BD(void)
 
   game_bd.time_left = time_left;
   game_bd.gems_still_needed = gems_still_needed;
-  game_bd.score = game_bd.game->player_score;
+  game_bd.score = game_bd.game->cave_score;			// use cave score here
 
   if (game.no_level_time_limit)
     game_bd.time_left = getTimePlayed_BD();
@@ -295,7 +303,7 @@ static void UpdateGameDoorValues_BD(void)
   {
     // update time and score in panel while counting bonus time
     game.LevelSolved_CountingTime  = game_bd.time_left;
-    game.LevelSolved_CountingScore = game_bd.score;
+    game.LevelSolved_CountingScore = game_bd.global_score;	// use global score here
   }
 }
 
@@ -407,21 +415,21 @@ void GameActions_BD(byte action[MAX_PLAYERS])
 		     player_x,
 		     player_y);
 
+  play_game_func(game_bd.game, action[0]);
+
   if (setup.bd_skip_hatching && !game_bd.game->cave->hatched &&
       game_bd.game->state_counter == GAME_INT_CAVE_RUNNING)
   {
     // fast-forward game engine until player hatched
     while (!game_bd.game->cave->hatched)
     {
-      play_game_func(game_bd.game, 0);
-
       // also record or replay tape action during fast-forward
       action = TapeCorrectAction_BD(action);
+
+      play_game_func(game_bd.game, 0);
     }
   }
 
-  play_game_func(game_bd.game, action[0]);
-
   // scroll without iterating engine if player out of sight (mainly due to wrap-around)
   // (this is needed to prevent broken tapes in case of viewport or tile size changes)
   while (game_bd.game->out_of_window)
@@ -518,9 +526,6 @@ void CoverScreen_BD(void)
 
     BackToFront();
   }
-
-  // stop uncovering loop sound when not using native sound engine
-  FadeSounds();
 }
 
 void BlitScreenToBitmap_BD(Bitmap *target_bitmap)
